
!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:17'!
abstractBytecodeMessagesDo: aBlock
	"Evaluate aBlock with the sequence of abstract bytecodes in the receiver."
	self method
		abstractBytecodeMessagesFrom: startpc
		to: self endPC
		do: aBlock! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:27'!
blockCreationBytecodeMessage
	"Answer the abstract bytecode message that created the receiver."
	| blockCreationBytecodeSize |
	^self method abstractBytecodeMessageAt: startpc - (blockCreationBytecodeSize := 4)

	"(SortedCollection sortBlock: [:a :b| a compare: b caseSensitive: false]) sortBlock blockCreationBytecodeMessage"! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:27'!
endPC
	^self blockCreationBytecodeMessage arguments last + startpc - 1! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 15:12'!
fuelValueWithoutNotifications
	SystemChangeNotifier uniqueInstance doSilently: self! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:16'!
isClean
	"Answer if the receiver does not close-over any variables other than globals, and does
	 not ^-return (does not close over the home context).  Clean blocks are amenable to
	 being created at compile-time."
	self numCopiedValues > 0 ifTrue:
		[^false].
	self abstractBytecodeMessagesDo:
		[:msg|
		(#(	pushReceiver
			pushReceiverVariable: popIntoReceiverVariable: storeIntoReceiverVariable:
			methodReturnConstant: methodReturnReceiver methodReturnTop)
				includes: msg selector) ifTrue:
					[^false]].
	^true! !

!Date class methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 10/18/2012 10:32'!
materializeFrom: aDecoder
	| start |
	start := (DateAndTime julianDayNumber: aDecoder nextEncodedInt32) midnight.
	start := start offset: aDecoder nextEncodedInt32 hours.
	^ self 
		starting: start
		duration: (Duration days: 1)! !

!Date methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 10/17/2012 20:10'!
serializeOn: anEncoder

	anEncoder 
		encodeInt32: self julianDayNumber;
		encodeInt32: self start offset hours! !

!Metaclass methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 15:01'!
addInstVarNamed: aString 
	"Add the argument, aString, as one of the receiver's instance variables."

	| fullString |
	fullString := String streamContents: [:strm |
		self instVarNames do: [:aString2 | strm nextPutAll: aString2; space].
		strm nextPutAll: aString].
	self instanceVariableNames: fullString! !

!Metaclass methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 15:46'!
removeInstVarNamed: aString 
	"Remove the argument, aString, as one of the receiver's instance variables."

	| newArray newString |
	(self instVarNames includes: aString)
		ifFalse: [self error: aString , ' is not one of my instance variables'].
	newArray := self instVarNames copyWithout: aString.
	newString := ''.
	newArray do: [:aString2 | newString := aString2 , ' ' , newString].
	self instanceVariableNames: newString! !

!SequenceableCollection methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 14:57'!
reduceRight: aBlock
	"Fold the result of the receiver from right to left into aBlock. The argument aBlock must take two or more arguments."

	"#(1 2 3) reduceRight: [ :a :b | a - b ].
		--> (1 - (2 - 3)) = 2
	#(1 + 3 - 5) reduceRight: [ :a :op :b | a perform: op with: b ].
		--> (1 + (3 - 5)) = -1"

	| arguments |
	self emptyCheck.
	arguments := Array new: aBlock argumentCount.
	(arguments size = 0 or: [ (self size + 1) \\ (arguments size - 1) > 0 ])
		ifTrue: [ self error: 'Collection size and block argument count do not match.' ].
	arguments at: arguments size put: self last.
	self size - arguments size + 1 to: 1 by: 1 - arguments size do: [ :index |
		arguments
			replaceFrom: 1 to: aBlock argumentCount - 1 with: self startingAt: index;
			at: arguments size put: (aBlock valueWithArguments: arguments) ].
	^ arguments last! !

!SequenceableCollection methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 14:58'!
sum: aBlock
	"This is implemented using a variant of the normal inject:into: pattern. 
	The reason for this is that it is not known whether we're in the normal 
	number line, i.e. whether 0 is a good initial value for the sum. 
	Consider a collection of measurement objects, 0 would be the unitless 
	value and would not be appropriate to add with the unit-ed objects."
	| sum sample |
	sample := aBlock value: self anyOne.
	sum := self inject: sample into: [ :previousValue :each |  previousValue + (aBlock value: each) ].
	^ sum - sample! !

!ClassDescription methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 14:59'!
instVarIndexFor: instVarName
	"Answer the index of the named instance variable."

	| index |
	index := instanceVariables == nil 
		ifTrue: [0]
		ifFalse: [instanceVariables indexOf: instVarName].
	index = 0 ifTrue: 
		[^superclass == nil 
			ifTrue: [0]
			ifFalse: [superclass instVarIndexFor: instVarName]].
	^superclass == nil 	
		ifTrue: [index]
		ifFalse: [index + superclass instSize]! !

!MethodContext methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:53'!
tempNamed: aName
	^self tempAt: (self tempNames indexOf: aName)! !

!MethodContext methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:54'!
tempNamed: aName put: anObject
	^self tempAt: (self tempNames indexOf: aName) put: anObject! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:26'!
abstractBytecodeMessageAt: pc
	"Answer the abstract bytecode message at pc in the receiver."
	^[(InstructionStream new method: self pc: pc) interpretNextInstructionFor: nil]
		on: MessageNotUnderstood
		do: [:ex| ex message]! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:15'!
abstractBytecodeMessagesFrom: startpc to: endpc do: aBlock
	"Evaluate aBlock with the sequence of abstract bytecodes from startpc through endpc in the receiver"
	| scanner |
	scanner := InstructionStream new method: self pc: startpc.
	[scanner pc <= endpc] whileTrue:
		[[scanner interpretNextInstructionFor: nil]
			on: MessageNotUnderstood
			do: [:ex| aBlock value: ex message]]! !

!ContextPart methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 16:30'!
arguments
	"returns the arguments of a message invocation"
	
	| arguments numargs |
	numargs :=  self method numArgs.
	arguments := Array new: numargs.
	1 to: numargs do: [:i | arguments at: i put: (self tempAt: i) ].
	^ arguments! !

!Class methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 15:01'!
addInstVarNamed: aString
	"Add the argument, aString, as one of the receiver's instance variables."
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: self instanceVariablesString, ' ', aString
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category
! !

!Class methodsFor: '*FuelCompatibilityBeforeSqueak45' stamp: 'MaxLeske 12/29/2012 15:46'!
removeInstVarNamed: aString 
	"Remove the argument, aString, as one of the receiver's instance variables."

	| newInstVarString |
	(self instVarNames includes: aString)
		ifFalse: [self error: aString , ' is not one of my instance variables'].
	newInstVarString := ''.
	(self instVarNames copyWithout: aString) do: 
		[:varName | newInstVarString := newInstVarString , ' ' , varName].
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: newInstVarString
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category! !
